@@ -22,18 +22,23 @@ import consolemd
 
 from substra import __version__, runner
 from substra.cli import printers
-from substra.sdk import assets, exceptions
+from substra.sdk import assets, exceptions, utils
 from substra.sdk import config as configuration
-from substra.sdk.client import Client
-from substra.sdk import user as usr
+from substra.sdk.client import Client, DEFAULT_BATCH_SIZE
+
+DEFAULT_RETRY_TIMEOUT = 300
 
 
 def get_client(global_conf):
-    """Initialize substra client from config file, profile name and user file."""
+    """Initialize substra client from config file, profile name and tokens file."""
     help_command = "substra config <url> ..."
 
     try:
-        client = Client(global_conf.config, global_conf.profile, global_conf.user)
+        client = Client.from_config_file(
+            config_path=global_conf.config,
+            profile_name=global_conf.profile,
+            tokens_path=global_conf.tokens,
+        )
 
     except FileNotFoundError:
         raise click.ClickException(
@@ -71,9 +76,16 @@ class GlobalConf:
     def __init__(self):
         self.profile = None
         self.config = None
-        self.user = None
+        self.tokens = None
         self.verbose = None
         self.output_format = None
+        self.retry_timeout = DEFAULT_RETRY_TIMEOUT
+
+    def retry(self, func):
+        return utils.retry_on_exception(
+            exceptions=(exceptions.RequestTimeout),
+            timeout=float(self.retry_timeout),
+        )(func)
 
 
 def update_global_conf(ctx, param, value):
@@ -90,15 +102,15 @@ def click_global_conf_profile(f):
         help='Profile name to use.')(f)
 
 
-def click_global_conf_user(f):
-    """Add user option to command."""
+def click_global_conf_tokens(f):
+    """Add tokens option to command."""
     return click.option(
-        '--user',
+        '--tokens',
         expose_value=False,
         type=click.Path(dir_okay=False, resolve_path=True),
         callback=update_global_conf,
-        default=usr.DEFAULT_PATH,
-        help='User file path to use (default ~/.substra-user).')(f)
+        default=os.path.expanduser(configuration.DEFAULT_TOKENS_PATH),
+        help=f'Tokens file path to use (default {configuration.DEFAULT_TOKENS_PATH}).')(f)
 
 
 def click_global_conf_config(f):
@@ -108,8 +120,8 @@ def click_global_conf_config(f):
         expose_value=False,
         type=click.Path(exists=True, resolve_path=True),
         callback=update_global_conf,
-        default=os.path.expanduser('~/.substra'),
-        help='Config path (default ~/.substra).')(f)
+        default=os.path.expanduser(configuration.DEFAULT_PATH),
+        help=f'Config path (default {configuration.DEFAULT_PATH}).')(f)
 
 
 def click_global_conf_verbose(f):
@@ -154,7 +166,7 @@ def click_global_conf_output_format(f):
 
 def click_global_conf(f):
     f = click_global_conf_verbose(f)
-    f = click_global_conf_user(f)
+    f = click_global_conf_tokens(f)
     f = click_global_conf_profile(f)
     f = click_global_conf_config(f)
     f = click_global_conf_log_level(f)
@@ -186,6 +198,19 @@ def click_option_expand(f):
     )(f)
 
 
+def click_global_conf_retry_timeout(f):
+    """Add timeout option to command."""
+    return click.option(
+        '--timeout', 'timeout',
+        type=click.INT,
+        expose_value=False,
+        default=DEFAULT_RETRY_TIMEOUT,
+        show_default=True,
+        callback=update_global_conf,
+        help='Max number of seconds the operation will be retried for'
+    )(f)
+
+
 def validate_json(ctx, param, value):
     if not value:
         return value
@@ -204,6 +229,30 @@ def load_data_samples_keys(data_samples, option="--data-samples-path"):
         raise click.BadParameter('File must contain a "keys" attribute.', param_hint=f'"{option}"')
 
 
+def _format_server_errors(fn, errors):
+    action = fn.__name__.replace('_', ' ')
+
+    def _format_error_lines(errors_):
+        lines_ = []
+        for field, field_errors in errors_.items():
+            for field_error in field_errors:
+                lines_.append(f"{field}: {field_error}")
+        return lines_
+
+    lines = []
+    if isinstance(errors, dict):
+        lines += _format_error_lines(errors)
+    elif isinstance(errors, list):
+        for error in errors:
+            lines += _format_error_lines(error)
+    else:
+        lines.append(errors)
+
+    pluralized_error = 'errors' if len(lines) > 1 else 'error'
+    return f"Could not {action}, the server returned the following \
+        {pluralized_error}:\n- " + '\n- '.join(lines)
+
+
 def error_printer(fn):
     """Command decorator to pretty print a few selected exceptions from sdk."""
     @functools.wraps(fn)
@@ -219,19 +268,11 @@ def error_printer(fn):
             raise click.ClickException('Login failed: No active account found with the'
                                        ' given credentials.')
         except exceptions.InvalidRequest as e:
-            if not isinstance(e.errors, dict):
-                raise click.ClickException(f"Request failed: {e.__class__.__name__}: {e}")
-
-            lines = []
-            for field, errors in e.errors.items():
-                for error in errors:
-                    lines.append(f"- {field}: {error}")
-
-            action = fn.__name__.replace('_', ' ')
-            pluralized_error = 'errors' if len(lines) > 1 else 'error'
-            lines.insert(0, f"Could not {action},"
-                            f"the server returned the following {pluralized_error}:")
-            raise click.ClickException("\n".join(lines))
+            try:
+                errors = e.errors['message']
+            except KeyError:
+                errors = e.errors
+            raise click.ClickException(_format_server_errors(fn, errors))
         except exceptions.RequestException as e:
             raise click.ClickException(f"Request failed: {e.__class__.__name__}: {e}")
         except (exceptions.ConnectionError,
@@ -258,21 +299,17 @@ def cli(ctx):
 @cli.command('config')
 @click.argument('url')
 @click.option('--config', type=click.Path(),
-              default=configuration.DEFAULT_PATH,
-              help='Config path (default ~/.substra).')
+              default=os.path.expanduser(configuration.DEFAULT_PATH),
+              help=f'Config path (default {configuration.DEFAULT_PATH}).')
 @click.option('--profile', default='default',
               help='Profile name to add')
 @click.option('--insecure', '-k', is_flag=True,
               help='Do not verify SSL certificates')
-@click.option('--version', '-v', default=configuration.DEFAULT_VERSION)
-def add_profile_to_config(url, config, profile, insecure, version):
+def add_profile_to_config(url, config, profile, insecure):
     """Add profile to config file."""
-    configuration.Manager(config).add_profile(
-        profile,
-        url,
-        version=version,
-        insecure=insecure,
-    )
+    manager = configuration.ConfigManager(config)
+    manager.set_profile(name=profile, url=url, insecure=insecure)
+    manager.save()
 
 
 @cli.command('login')
@@ -283,12 +320,15 @@ def add_profile_to_config(url, config, profile, insecure, version):
 @click.option('--password', '-p', envvar='SUBSTRA_PASSWORD', prompt=True, hide_input=True)
 def login(ctx, username, password):
     """Login to the Substra platform."""
-    usr.Manager(ctx.obj.user).clear_user()
     client = get_client(ctx.obj)
 
     token = client.login(username, password)
-    # create temporary user data
-    usr.Manager(ctx.obj.user).add_user(token)
+
+    # save token to tokens file
+    manager = configuration.TokenManager(ctx.obj.tokens)
+    manager.set_profile(ctx.obj.profile, token)
+    manager.save()
+
     display(f"Token: {token}")
 
 
@@ -309,6 +349,7 @@ def add(ctx):
 @click.option('--test-only', is_flag=True, default=False,
               help='Data sample(s) used as test data only.')
 @click_global_conf
+@click_global_conf_retry_timeout
 @click.pass_context
 @error_printer
 def add_data_sample(ctx, path, dataset_key, local, multiple, test_only):
@@ -336,8 +377,8 @@ def add_data_sample(ctx, path, dataset_key, local, multiple, test_only):
     }
     if test_only:
         data['test_only'] = True
-    res = client.add_data_samples(data, local=local)
-    display(res)
+    key = client.add_data_samples(data, local=local)
+    display(key)
 
 
 @add.command('dataset')
@@ -345,6 +386,7 @@ def add_data_sample(ctx, path, dataset_key, local, multiple, test_only):
                 metavar="PATH")
 @click.option('--objective-key')
 @click_global_conf_with_output_format
+@click_global_conf_retry_timeout
 @click.pass_context
 @error_printer
 def add_dataset(ctx, data, objective_key):
@@ -379,7 +421,8 @@ def add_dataset(ctx, data, objective_key):
     if objective_key:  # overwrite data values if set
         data['objective_key'] = objective_key
 
-    res = client.add_dataset(data)
+    key = client.add_dataset(data)
+    res = ctx.obj.retry(client.get_dataset)(key)
     printer = printers.get_asset_printer(assets.DATASET, ctx.obj.output_format)
     printer.print(res, is_list=False)
 
@@ -392,6 +435,7 @@ def add_dataset(ctx, data, objective_key):
               type=click.Path(exists=True, resolve_path=True, dir_okay=False),
               callback=load_json_from_path, help='Test data samples.')
 @click_global_conf_with_output_format
+@click_global_conf_retry_timeout
 @click.pass_context
 @error_printer
 def add_objective(ctx, data, dataset_key, data_samples):
@@ -441,7 +485,8 @@ def add_objective(ctx, data, dataset_key, data_samples):
     if data_samples:
         data['test_data_sample_keys'] = load_data_samples_keys(data_samples)
 
-    res = client.add_objective(data)
+    key = client.add_objective(data)
+    res = ctx.obj.retry(client.get_objective)(key)
     printer = printers.get_asset_printer(assets.OBJECTIVE, ctx.obj.output_format)
     printer.print(res, is_list=False)
 
@@ -450,6 +495,7 @@ def add_objective(ctx, data, dataset_key, data_samples):
 @click.argument('data', type=click.Path(exists=True, dir_okay=False), callback=load_json_from_path,
                 metavar="PATH")
 @click_global_conf_with_output_format
+@click_global_conf_retry_timeout
 @click.pass_context
 @error_printer
 def add_algo(ctx, data):
@@ -479,7 +525,8 @@ def add_algo(ctx, data):
     """
 
     client = get_client(ctx.obj)
-    res = client.add_algo(data)
+    key = client.add_algo(data)
+    res = ctx.obj.retry(client.get_algo)(key)
     printer = printers.get_asset_printer(assets.ALGO, ctx.obj.output_format)
     printer.print(res, is_list=False)
 
@@ -487,10 +534,15 @@ def add_algo(ctx, data):
 @add.command('compute_plan')
 @click.argument('data', type=click.Path(exists=True, dir_okay=False),
                 callback=load_json_from_path, metavar="PATH")
+@click.option('--no-auto-batching', '-n', is_flag=True,
+              help='Disable the auto batching feature')
+@click.option('--batch-size', '-b', type=int,
+              help='Batch size for the auto batching',
+              default=DEFAULT_BATCH_SIZE, show_default=True)
 @click_global_conf_with_output_format
 @click.pass_context
 @error_printer
-def add_compute_plan(ctx, data):
+def add_compute_plan(ctx, data, no_auto_batching, batch_size):
     """Add compute plan.
 
     The path must point to a valid JSON file with the following schema:
@@ -540,9 +592,17 @@ def add_compute_plan(ctx, data):
         "metadata": dict
     }
 
+    Disable the auto batching to upload all the tuples of the
+    compute plan at once.
+    If the auto batching is enabled, change the `batch_size` to define the number of
+    tuples uploaded in each batch (default 20).
     """
+    if no_auto_batching and batch_size:
+        raise click.BadOptionUsage('--batch_size',
+                                   "The --batch_size option cannot be used when using "
+                                   "--no_auto_batching.")
     client = get_client(ctx.obj)
-    res = client.add_compute_plan(data)
+    res = client.add_compute_plan(data, auto_batching=not no_auto_batching, batch_size=batch_size)
     printer = printers.get_asset_printer(assets.COMPUTE_PLAN, ctx.obj.output_format)
     printer.print(res, is_list=False)
 
@@ -551,6 +611,7 @@ def add_compute_plan(ctx, data):
 @click.argument('data', type=click.Path(exists=True, dir_okay=False), callback=load_json_from_path,
                 metavar="PATH")
 @click_global_conf_with_output_format
+@click_global_conf_retry_timeout
 @click.pass_context
 @error_printer
 def add_aggregate_algo(ctx, data):
@@ -580,7 +641,8 @@ def add_aggregate_algo(ctx, data):
     """
 
     client = get_client(ctx.obj)
-    res = client.add_aggregate_algo(data)
+    key = client.add_aggregate_algo(data)
+    res = ctx.obj.retry(client.get_aggregate_algo)(key)
     printer = printers.get_asset_printer(assets.AGGREGATE_ALGO, ctx.obj.output_format)
     printer.print(res, is_list=False)
 
@@ -589,6 +651,7 @@ def add_aggregate_algo(ctx, data):
 @click.argument('data', type=click.Path(exists=True, dir_okay=False), callback=load_json_from_path,
                 metavar="PATH")
 @click_global_conf_with_output_format
+@click_global_conf_retry_timeout
 @click.pass_context
 @error_printer
 def add_composite_algo(ctx, data):
@@ -618,7 +681,8 @@ def add_composite_algo(ctx, data):
     """
 
     client = get_client(ctx.obj)
-    res = client.add_composite_algo(data)
+    key = client.add_composite_algo(data)
+    res = ctx.obj.retry(client.get_composite_algo)(key)
     printer = printers.get_asset_printer(assets.COMPOSITE_ALGO, ctx.obj.output_format)
     printer.print(res, is_list=False)
 
@@ -633,6 +697,7 @@ def add_composite_algo(ctx, data):
               help='In model traintuple key.')
 @click.option('--tag')
 @click_global_conf_with_output_format
+@click_global_conf_retry_timeout
 @click_option_metadata
 @click.pass_context
 @error_printer
@@ -668,7 +733,8 @@ def add_traintuple(ctx, algo_key, dataset_key, data_samples, in_models_keys, tag
 
     if in_models_keys:
         data['in_models_keys'] = in_models_keys
-    res = client.add_traintuple(data)
+    key = client.add_traintuple(data)
+    res = ctx.obj.retry(client.get_traintuple)(key)
     printer = printers.get_asset_printer(assets.TRAINTUPLE, ctx.obj.output_format)
     printer.print(res, is_list=False)
 
@@ -681,6 +747,7 @@ def add_traintuple(ctx, algo_key, dataset_key, data_samples, in_models_keys, tag
 @click.option('--rank', type=click.INT)
 @click.option('--tag')
 @click_global_conf_with_output_format
+@click_global_conf_retry_timeout
 @click_option_metadata
 @click.pass_context
 @error_printer
@@ -703,7 +770,8 @@ def add_aggregatetuple(ctx, algo_key, in_models_keys, worker, rank, tag, metadat
 
     if metadata:
         data['metadata'] = metadata
-    res = client.add_aggregatetuple(data)
+    key = client.add_aggregatetuple(data)
+    res = ctx.obj.retry(client.get_aggregatetuple)(key)
     printer = printers.get_asset_printer(assets.AGGREGATETUPLE, ctx.obj.output_format)
     printer.print(res, is_list=False)
 
@@ -724,6 +792,7 @@ def add_aggregatetuple(ctx, algo_key, in_models_keys, worker, rank, tag, metadat
               help='Load a permissions file.')
 @click.option('--tag')
 @click_global_conf_with_output_format
+@click_global_conf_retry_timeout
 @click_option_metadata
 @click.pass_context
 @error_printer
@@ -780,7 +849,8 @@ def add_composite_traintuple(ctx, algo_key, dataset_key, data_samples, head_mode
 
     if metadata:
         data['metadata'] = metadata
-    res = client.add_composite_traintuple(data)
+    key = client.add_composite_traintuple(data)
+    res = ctx.obj.retry(client.get_composite_traintuple)(key)
     printer = printers.get_asset_printer(assets.COMPOSITE_TRAINTUPLE, ctx.obj.output_format)
     printer.print(res, is_list=False)
 
@@ -794,6 +864,7 @@ def add_composite_traintuple(ctx, algo_key, dataset_key, data_samples, head_mode
               callback=load_json_from_path)
 @click.option('--tag')
 @click_global_conf_with_output_format
+@click_global_conf_retry_timeout
 @click_option_metadata
 @click.pass_context
 @error_printer
@@ -827,7 +898,8 @@ def add_testtuple(ctx, objective_key, dataset_key, traintuple_key, data_samples,
 
     if metadata:
         data['metadata'] = metadata
-    res = client.add_testtuple(data)
+    key = client.add_testtuple(data)
+    res = ctx.obj.retry(client.get_testtuple)(key)
     printer = printers.get_asset_printer(assets.TESTTUPLE, ctx.obj.output_format)
     printer.print(res, is_list=False)
 
@@ -918,7 +990,8 @@ def list_(ctx, asset_name, filters, filters_logical_clause, advanced_filters):
         filters = advanced_filters
     res = method(filters)
     printer = printers.get_asset_printer(asset_name, ctx.obj.output_format)
-    printer.print(res, is_list=True)
+    dict_res = [result.dict(exclude_none=False, by_alias=True) for result in res]
+    printer.print(dict_res, is_list=True)
 
 
 @cli.command()
@@ -1023,9 +1096,12 @@ def leaderboard(ctx, expand, objective_key, sort):
 @click.option('--fake-data-samples',
               is_flag=True,
               help='use fake data samples during both training and testing.')
+@click.option('--n-fake-samples',
+              type=click.INT,
+              help='Number of fake data samples requested')
 def run_local(algo, train_opener, test_opener, metrics, rank,
               train_data_samples, test_data_samples, inmodels,
-              fake_data_samples):
+              fake_data_samples, n_fake_samples):
     """Run local.
 
     Train and test the algo located in ALGO (directory or archive) locally.
@@ -1060,6 +1136,10 @@ def run_local(algo, train_opener, test_opener, metrics, rank,
     if not fake_data_samples and not train_data_samples and test_data_samples:
         raise click.BadOptionUsage('--train-data-samples',
                                    'Missing option --train-data-samples')
+    if not fake_data_samples and n_fake_samples:
+        raise click.BadOptionUsage('--n-fake-samples',
+                                   'Option --n-fake-samples cannot be used if'
+                                   '--fake-data-samples is not activated')
 
     try:
         runner.compute(algo_path=algo,
@@ -1070,7 +1150,8 @@ def run_local(algo, train_opener, test_opener, metrics, rank,
                        test_data_path=test_data_samples,
                        fake_data_samples=fake_data_samples,
                        rank=rank,
-                       inmodels=inmodels)
+                       inmodels=inmodels,
+                       n_fake_samples=n_fake_samples)
     except runner.PathTraversalException as e:
         raise click.ClickException(
             f'Archive "{e.archive_path}" includes at least 1 file or folder '
@@ -1150,10 +1231,15 @@ def update_dataset(ctx, dataset_key, objective_key):
 @click.argument('compute_plan_id', type=click.STRING)
 @click.argument('tuples', type=click.Path(exists=True, dir_okay=False),
                 callback=load_json_from_path, metavar="TUPLES_PATH")
+@click.option('--no-auto-batching', '-n', is_flag=True,
+              help='Disable the auto batching feature')
+@click.option('--batch-size', '-b', type=int,
+              help='Batch size for the auto batching',
+              default=DEFAULT_BATCH_SIZE, show_default=True)
 @click_global_conf_with_output_format
 @click.pass_context
 @error_printer
-def update_compute_plan(ctx, compute_plan_id, tuples):
+def update_compute_plan(ctx, compute_plan_id, tuples, no_auto_batching, batch_size):
     """Update compute plan.
 
     The tuples path must point to a valid JSON file with the following schema:
@@ -1200,9 +1286,17 @@ def update_compute_plan(ctx, compute_plan_id, tuples):
         }]
     }
 
+    Disable the auto batching to upload all the tuples of the
+    compute plan at once.
+    If the auto batching is enabled, change the `batch_size` to define the number of
+    tuples uploaded in each batch (default 20).
     """
+    if no_auto_batching and batch_size:
+        raise click.BadOptionUsage('--batch_size',
+                                   "The --batch_size option cannot be used when using "
+                                   "--no_auto_batching.")
     client = get_client(ctx.obj)
-    res = client.update_compute_plan(compute_plan_id, tuples)
+    res = client.update_compute_plan(compute_plan_id, tuples, not no_auto_batching, batch_size)
     printer = printers.get_asset_printer(assets.COMPUTE_PLAN, ctx.obj.output_format)
     printer.print(res, is_list=False)
 
