@@ -14,6 +14,7 @@
 
 import json
 import math
+import pydantic
 
 import yaml
 
@@ -41,6 +42,8 @@ class Field:
     def print_details(self, item, field_length, expand):
         name = self.name.upper().ljust(field_length)
         value = self.get_value(item, expand)
+        if isinstance(value, dict):
+            value = [f'{k}: {v}' for k, v in value.items()]
 
         if isinstance(value, list):
             if value:
@@ -63,7 +66,7 @@ class PermissionField(Field):
         if is_public:
             return 'Processable by anyone'
 
-        authorized_ids = find_dict_composite_key_value(item, f'{self.ref}.process.authorizedIDs')
+        authorized_ids = find_dict_composite_key_value(item, f'{self.ref}.process.authorized_ids')
         if not authorized_ids:
             return 'Processable by its owner only'
 
@@ -105,7 +108,7 @@ class CountField(Field):
 
 class InModelTraintupleKeysField(KeysField):
     def _get_key(self, v):
-        return v.get('traintupleKey')
+        return v.get('traintuple_key')
 
 
 class CurrentNodeField(Field):
@@ -176,6 +179,8 @@ class BasePrinter:
         return length
 
     def print_details(self, item, fields, expand):
+        if isinstance(item, pydantic.BaseModel):
+            item = item.dict(exclude_none=False, by_alias=True)
         field_length = self._get_field_name_length(fields)
         for field in fields:
             field.print_details(item, field_length, expand)
@@ -221,6 +226,8 @@ class AssetPrinter(BasePrinter):
         self.print_description_message(item, profile)
 
     def print(self, data, profile=None, expand=False, is_list=False):
+        if isinstance(data, pydantic.BaseModel):
+            data = data.dict(exclude_none=False, by_alias=True)
         if is_list:
             self.print_table(data, self._get_list_fields())
         else:
@@ -247,6 +254,7 @@ class BaseAlgoPrinter(AssetPrinter):
     single_fields = (
         Field('Name', 'name'),
         Field('Owner', 'owner'),
+        Field('Metadata', 'metadata'),
         PermissionField('Permissions', 'permissions'),
     )
 
@@ -256,28 +264,29 @@ class BaseAlgoPrinter(AssetPrinter):
 class ComputePlanPrinter(AssetPrinter):
     asset_name = 'compute_plan'
 
-    key_field = Field('Compute plan ID', 'computePlanID')
+    key_field = Field('Compute plan ID', 'compute_plan_id')
 
     list_fields = (
-        CountField('Traintuples count', 'traintupleKeys'),
-        CountField('Composite traintuples count', 'compositeTraintupleKeys'),
-        CountField('Aggregatetuples count', 'aggregatetupleKeys'),
-        CountField('Testtuples count', 'testtupleKeys'),
-        ProgressField('Progress', 'doneCount', 'tupleCount'),
+        CountField('Traintuples count', 'traintuple_keys'),
+        CountField('Composite traintuples count', 'composite_traintuple_keys'),
+        CountField('Aggregatetuples count', 'aggregatetuple_keys'),
+        CountField('Testtuples count', 'testtuple_keys'),
+        ProgressField('Progress', 'done_count', 'tuple_count'),
         Field('Status', 'status'),
         Field('Tag', 'tag'),
         Field('Clean model', 'clean_model'),
     )
     single_fields = (
-        KeysField('Traintuple keys', 'traintupleKeys'),
-        KeysField('Composite traintuple keys', 'compositeTraintupleKeys'),
-        KeysField('Aggregatetuple keys', 'aggregatetupleKeys'),
-        KeysField('Testtuple keys', 'testtupleKeys'),
-        ProgressField('Progress', 'doneCount', 'tupleCount'),
+        KeysField('Traintuple keys', 'traintuple_keys'),
+        KeysField('Composite traintuple keys', 'composite_traintuple_keys'),
+        KeysField('Aggregatetuple keys', 'aggregatetuple_keys'),
+        KeysField('Testtuple keys', 'testtuple_keys'),
+        ProgressField('Progress', 'done_count', 'tuple_count'),
         Field('Status', 'status'),
         Field('Tag', 'tag'),
+        Field('Metadata', 'metadata'),
         Field('Clean model', 'clean_model'),
-        MappingField('ID to key mapping', 'IDToKey'),
+        MappingField('ID to key mapping', 'id_to_key'),
     )
 
     def print_messages(self, item, profile=None):
@@ -285,19 +294,20 @@ class ComputePlanPrinter(AssetPrinter):
         profile_arg = self.get_profile_arg(profile)
 
         print('\nDisplay this compute_plan\'s traintuples:')
-        print(f'\tsubstra list traintuple -f "traintuple:computePlanID:{key_value}" {profile_arg}')
+        print(f'\tsubstra list traintuple -f '
+              f'"traintuple:compute_plan_id:{key_value}" {profile_arg}')
 
         print('\nDisplay this compute_plan\'s composite_traintuples:')
         print(f'\tsubstra list composite_traintuple'
-              f' -f "composite_traintuple:computePlanID:{key_value}" {profile_arg}')
+              f' -f "composite_traintuple:compute_plan_id:{key_value}" {profile_arg}')
 
         print('\nDisplay this compute_plan\'s aggregatetuples:')
         print(f'\tsubstra list aggregatetuple'
-              f' -f "aggregatetuple:computePlanID:{key_value}" {profile_arg}')
+              f' -f "aggregatetuple:compute_plan_id:{key_value}" {profile_arg}')
 
         print('\nDisplay this compute_plan\'s testtuples:')
         print(f'\tsubstra list testtuple'
-              f' -f "testtuple:computePlanID:{key_value}" {profile_arg}')
+              f' -f "testtuple:compute_plan_id:{key_value}" {profile_arg}')
 
 
 class AlgoPrinter(BaseAlgoPrinter):
@@ -322,9 +332,10 @@ class ObjectivePrinter(AssetPrinter):
     single_fields = (
         Field('Name', 'name'),
         Field('Metrics', 'metrics.name'),
-        Field('Test dataset key', 'testDataset.dataManagerKey'),
-        KeysField('Test data sample keys', 'testDataset.dataSampleKeys'),
+        Field('Test dataset key', 'test_dataset.data_manager_key'),
+        KeysField('Test data sample keys', 'test_dataset.data_sample_keys'),
         Field('Owner', 'owner'),
+        Field('Metadata', 'metadata'),
         PermissionField('Permissions', 'permissions'),
     )
     download_message = 'Download this objective\'s metric:'
@@ -353,11 +364,12 @@ class DatasetPrinter(AssetPrinter):
     )
     single_fields = (
         Field('Name', 'name'),
-        Field('Objective key', 'objectiveKey'),
+        Field('Objective key', 'objective_key'),
         Field('Type', 'type'),
-        KeysField('Train data sample keys', 'trainDataSampleKeys'),
-        KeysField('Test data sample keys', 'testDataSampleKeys'),
+        KeysField('Train data sample keys', 'train_data_sample_keys'),
+        KeysField('Test data sample keys', 'test_data_sample_keys'),
         Field('Owner', 'owner'),
+        Field('Metadata', 'metadata'),
         PermissionField('Permissions', 'permissions'),
     )
     download_message = 'Download this data manager\'s opener:'
@@ -371,22 +383,23 @@ class TraintuplePrinter(AssetPrinter):
         Field('Status', 'status'),
         Field('Rank', 'rank'),
         Field('Tag', 'tag'),
-        Field('Compute Plan Id', 'computePlanID'),
+        Field('Compute Plan Id', 'compute_plan_id'),
     )
     single_fields = (
-        Field('Model key', 'outModel.hash'),
+        Field('Model key', 'out_model.hash'),
         Field('Algo key', 'algo.hash'),
         Field('Algo name', 'algo.name'),
         Field('Status', 'status'),
-        Field('Dataset key', 'dataset.openerHash'),
+        Field('Dataset key', 'dataset.opener_hash'),
         KeysField('Train data sample keys', 'dataset.keys'),
-        InModelTraintupleKeysField('In model traintuple keys', 'inModels'),
+        InModelTraintupleKeysField('In model traintuple keys', 'in_models'),
         Field('Rank', 'rank'),
-        Field('Compute Plan Id', 'computePlanID'),
+        Field('Compute Plan Id', 'compute_plan_id'),
         Field('Tag', 'tag'),
         Field('Log', 'log'),
         Field('Creator', 'creator'),
         Field('Worker', 'dataset.worker'),
+        Field('Metadata', 'metadata'),
         PermissionField('Permissions', 'permissions'),
     )
     has_description = False
@@ -396,7 +409,7 @@ class TraintuplePrinter(AssetPrinter):
         profile_arg = self.get_profile_arg(profile)
 
         print('\nDisplay this traintuple\'s testtuples:')
-        print(f'\tsubstra list testtuple -f "testtuple:traintupleKey:{key_value}" {profile_arg}')
+        print(f'\tsubstra list testtuple -f "testtuple:traintuple_key:{key_value}" {profile_arg}')
 
 
 class AggregateTuplePrinter(AssetPrinter):
@@ -407,21 +420,22 @@ class AggregateTuplePrinter(AssetPrinter):
         Field('Status', 'status'),
         Field('Rank', 'rank'),
         Field('Tag', 'tag'),
-        Field('Compute Plan Id', 'computePlanID'),
+        Field('Compute Plan Id', 'compute_plan_id'),
     )
     single_fields = (
-        Field('Model key', 'outModel.hash'),
+        Field('Model key', 'out_model.hash'),
         Field('Algo key', 'algo.hash'),
         Field('Algo name', 'algo.name'),
         Field('Status', 'status'),
-        Field('Dataset key', 'dataset.openerHash'),
-        InModelTraintupleKeysField('In model keys', 'inModels'),
+        Field('Dataset key', 'dataset.opener_hash'),
+        InModelTraintupleKeysField('In model keys', 'in_models'),
         Field('Rank', 'rank'),
-        Field('Compute Plan Id', 'computePlanID'),
+        Field('Compute Plan Id', 'compute_plan_id'),
         Field('Tag', 'tag'),
         Field('Log', 'log'),
         Field('Creator', 'creator'),
         Field('Worker', 'dataset.worker'),
+        Field('Metadata', 'metadata'),
         PermissionField('Permissions', 'permissions'),
     )
     has_description = False
@@ -431,7 +445,7 @@ class AggregateTuplePrinter(AssetPrinter):
         profile_arg = self.get_profile_arg(profile)
 
         print('\nDisplay this aggregatetuple\'s testtuples:')
-        print(f'\tsubstra list testtuple -f "testtuple:traintupleKey:{key_value}" {profile_arg}')
+        print(f'\tsubstra list testtuple -f "testtuple:traintuple_key:{key_value}" {profile_arg}')
 
 
 class CompositeTraintuplePrinter(AssetPrinter):
@@ -442,27 +456,28 @@ class CompositeTraintuplePrinter(AssetPrinter):
         Field('Status', 'status'),
         Field('Rank', 'rank'),
         Field('Tag', 'tag'),
-        Field('Compute Plan Id', 'computePlanID'),
+        Field('Compute Plan Id', 'compute_plan_id'),
     )
 
     single_fields = (
-        Field('Out head model key', 'outHeadModel.outModel.hash'),
-        PermissionField('Out head model permissions', 'outHeadModel.permissions'),
-        Field('Out trunk model key', 'outTrunkModel.outModel.hash'),
-        PermissionField('Out trunk model permissions', 'outTrunkModel.permissions'),
+        Field('Out head model key', 'out_head_model.out_model.hash'),
+        PermissionField('Out head model permissions', 'out_head_model.permissions'),
+        Field('Out trunk model key', 'out_trunk_model.out_model.hash'),
+        PermissionField('Out trunk model permissions', 'out_trunk_model.permissions'),
         Field('Composite algo key', 'algo.hash'),
         Field('Composite algo name', 'algo.name'),
         Field('Status', 'status'),
-        Field('Dataset key', 'dataset.openerHash'),
+        Field('Dataset key', 'dataset.opener_hash'),
         KeysField('Train data sample keys', 'dataset.keys'),
-        Field('In head model key', 'inHeadModel.traintupleKey'),
-        Field('In trunk model key', 'inTrunkModel.traintupleKey'),
+        Field('In head model key', 'in_head_model.traintuple_key'),
+        Field('In trunk model key', 'in_trunk_model.traintuple_key'),
         Field('Rank', 'rank'),
-        Field('Compute Plan Id', 'computePlanID'),
+        Field('Compute Plan Id', 'compute_plan_id'),
         Field('Tag', 'tag'),
         Field('Log', 'log'),
         Field('Creator', 'creator'),
         Field('Worker', 'dataset.worker'),
+        Field('Metadata', 'metadata'),
     )
     has_description = False
 
@@ -471,7 +486,7 @@ class CompositeTraintuplePrinter(AssetPrinter):
         profile_arg = self.get_profile_arg(profile)
 
         print('\nDisplay this composite traintuple\'s testtuples:')
-        print(f'\tsubstra list testtuple -f "testtuple:traintupleKey:{key_value}" {profile_arg}')
+        print(f'\tsubstra list testtuple -f "testtuple:traintuple_key:{key_value}" {profile_arg}')
 
 
 class TesttuplePrinter(AssetPrinter):
@@ -484,22 +499,23 @@ class TesttuplePrinter(AssetPrinter):
         Field('Perf', 'dataset.perf'),
         Field('Rank', 'rank'),
         Field('Tag', 'tag'),
-        Field('Compute Plan Id', 'computePlanID'),
+        Field('Compute Plan Id', 'compute_plan_id'),
     )
     single_fields = (
-        Field('Traintuple key', 'traintupleKey'),
-        Field('Traintuple type', 'traintupleType'),
+        Field('Traintuple key', 'traintuple_key'),
+        Field('Traintuple type', 'traintuple_type'),
         Field('Algo key', 'algo.hash'),
         Field('Algo name', 'algo.name'),
         Field('Objective key', 'objective.hash'),
         Field('Certified', 'certified'),
         Field('Status', 'status'),
         Field('Perf', 'dataset.perf'),
-        Field('Dataset key', 'dataset.openerHash'),
+        Field('Dataset key', 'dataset.opener_hash'),
         KeysField('Test data sample keys', 'dataset.keys'),
         Field('Rank', 'rank'),
         Field('Tag', 'tag'),
-        Field('Compute Plan Id', 'computePlanID'),
+        Field('Metadata', 'metadata'),
+        Field('Compute Plan Id', 'compute_plan_id'),
         Field('Log', 'log'),
         Field('Creator', 'creator'),
         Field('Worker', 'dataset.worker'),
@@ -512,7 +528,7 @@ class NodePrinter(AssetPrinter):
     asset_name = 'node'
     key_field = Field('NODE ID', 'id')
     list_fields = (
-        CurrentNodeField('', 'isCurrent'),
+        CurrentNodeField('', 'is_current'),
     )
 
 
@@ -521,7 +537,7 @@ class LeaderBoardPrinter(BasePrinter):
     testtuple_fields = (
         Field('Perf', 'perf'),
         Field('Algo name', 'algo.name'),
-        Field('Traintuple key', 'traintupleKey'),
+        Field('Traintuple key', 'traintuple_key'),
     )
 
     def print(self, leaderboard, expand):
