@@ -15,7 +15,6 @@
 import contextlib
 import copy
 import io
-import itertools
 import functools
 import logging
 import time
@@ -87,9 +86,10 @@ def extract_data_sample_files(data):
         folders[attr] = data[attr]
         del data[attr]
 
-    for p in list(data.get('paths', [])):
-        folders[path_leaf(p)] = p
-        data['paths'].remove(p)
+    if data.get('paths'):  # field is set and is not None/empty
+        for p in data['paths']:
+            folders[path_leaf(p)] = p
+            data['paths'].remove(p)
 
     files = {}
     for k, f in folders.items():
@@ -104,14 +104,6 @@ def extract_data_sample_files(data):
             f.close()
 
 
-def flatten(list_of_list):
-    res = []
-    for item in itertools.chain.from_iterable(list_of_list):
-        if item not in res:
-            res.append(item)
-    return res
-
-
 def _join_and_groups(items):
     """
     "-OR-" items separate the items that have to be grouped with an "AND" clause
@@ -157,7 +149,26 @@ def parse_filters(filters):
 
 
 def retry_on_exception(exceptions, timeout=300):
-    """Retry function in case of exception(s)."""
+    """Retry function in case of exception(s).
+
+    Args:
+        exceptions (list): list of exception types that trigger a retry
+        timeout (int, optional): timeout in seconds
+
+    Example:
+        ```python
+        from substra.sdk import exceptions, retry_on_exception
+
+        def my_function(arg1, arg2):
+            pass
+
+        retry = retry_on_exception(
+                    exceptions=(exceptions.RequestTimeout),
+                    timeout=300,
+                )
+        retry(my_function)(arg1, arg2)
+        ```
+    """
     def _retry(f):
         @functools.wraps(f)
         def wrapper(*args, **kwargs):
