@@ -13,6 +13,9 @@
 # limitations under the License.
 import pytest
 import substra
+from substra.sdk import models
+
+import pydantic
 
 from .. import datastore
 from .utils import mock_requests
@@ -21,9 +24,10 @@ from .utils import mock_requests
 def test_add_dataset(client, dataset_query, mocker):
     m_post = mock_requests(mocker, "post", response=datastore.DATASET)
     m_get = mock_requests(mocker, "get", response=datastore.DATASET)
-    response = client.add_dataset(dataset_query)
+    key = client.add_dataset(dataset_query)
+    response = client.get_dataset(key)
 
-    assert response == datastore.DATASET
+    assert response == models.Dataset(**datastore.DATASET)
     m_post.assert_called()
     m_get.assert_called()
 
@@ -32,7 +36,7 @@ def test_add_dataset_invalid_args(client, dataset_query, mocker):
     mock_requests(mocker, "post", response=datastore.DATASET)
     del dataset_query['data_opener']
 
-    with pytest.raises(substra.sdk.exceptions.LoadDataException):
+    with pytest.raises(pydantic.error_wrappers.ValidationError):
         client.add_dataset(dataset_query)
 
 
@@ -44,20 +48,21 @@ def test_add_dataset_response_failure_500(client, dataset_query, mocker):
 
 
 def test_add_dataset_response_failure_409(client, dataset_query, mocker):
-    mock_requests(mocker, "post", response={"pkhash": "42"}, status=409)
+    mock_requests(mocker, "post", response={"key": "42"}, status=409)
 
     with pytest.raises(substra.sdk.exceptions.AlreadyExists) as exc_info:
         client.add_dataset(dataset_query)
 
-    assert exc_info.value.pkhash == "42"
+    assert exc_info.value.key == "42"
 
 
 def test_add_objective(client, objective_query, mocker):
     m_post = mock_requests(mocker, "post", response=datastore.OBJECTIVE)
     m_get = mock_requests(mocker, "get", response=datastore.OBJECTIVE)
-    response = client.add_objective(objective_query)
+    key = client.add_objective(objective_query)
+    response = client.get_objective(key)
 
-    assert response == datastore.OBJECTIVE
+    assert response == models.Objective(**datastore.OBJECTIVE)
     m_post.assert_called()
     m_get.assert_called()
 
@@ -65,9 +70,10 @@ def test_add_objective(client, objective_query, mocker):
 def test_add_algo(client, algo_query, mocker):
     m_post = mock_requests(mocker, "post", response=datastore.ALGO)
     m_get = mock_requests(mocker, "get", response=datastore.ALGO)
-    response = client.add_algo(algo_query)
+    key = client.add_algo(algo_query)
+    response = client.get_algo(key)
 
-    assert response == datastore.ALGO
+    assert response == models.Algo(**datastore.ALGO)
     m_post.assert_called()
     m_get.assert_called()
 
@@ -75,9 +81,10 @@ def test_add_algo(client, algo_query, mocker):
 def test_add_aggregate_algo(client, algo_query, mocker):
     m_post = mock_requests(mocker, "post", response=datastore.AGGREGATE_ALGO)
     m_get = mock_requests(mocker, "get", response=datastore.AGGREGATE_ALGO)
-    response = client.add_aggregate_algo(algo_query)
+    key = client.add_aggregate_algo(algo_query)
+    response = client.get_aggregate_algo(key)
 
-    assert response == datastore.AGGREGATE_ALGO
+    assert response == models.AggregateAlgo(**datastore.AGGREGATE_ALGO)
     m_post.assert_called()
     m_get.assert_called()
 
@@ -85,9 +92,10 @@ def test_add_aggregate_algo(client, algo_query, mocker):
 def test_add_composite_algo(client, algo_query, mocker):
     m_post = mock_requests(mocker, "post", response=datastore.COMPOSITE_ALGO)
     m_get = mock_requests(mocker, "get", response=datastore.COMPOSITE_ALGO)
-    response = client.add_composite_algo(algo_query)
+    key = client.add_composite_algo(algo_query)
+    response = client.get_composite_algo(key)
 
-    assert response == datastore.COMPOSITE_ALGO
+    assert response == models.CompositeAlgo(**datastore.COMPOSITE_ALGO)
     m_post.assert_called()
     m_get.assert_called()
 
@@ -97,15 +105,15 @@ def test_add_data_sample(client, data_sample_query, mocker):
     m = mock_requests(mocker, "post", response=server_response)
     response = client.add_data_sample(data_sample_query)
 
-    assert response == server_response[0]
+    assert response == server_response[0]['key']
     m.assert_called()
 
 
 def test_add_data_sample_already_exists(client, data_sample_query, mocker):
-    m = mock_requests(mocker, "post", response=[{"pkhash": "42"}], status=409)
+    m = mock_requests(mocker, "post", response=[{"key": "42"}], status=409)
     response = client.add_data_sample(data_sample_query, exist_ok=True)
 
-    assert response == {"pkhash": "42"}
+    assert response == "42"
     m.assert_called()
 
 
@@ -120,7 +128,7 @@ def test_add_data_samples(client, data_samples_query, mocker):
     m = mock_requests(mocker, "post", response=server_response)
     response = client.add_data_samples(data_samples_query)
 
-    assert response == server_response
+    assert response == ['42']
     m.assert_called()
 
 
