@@ -23,6 +23,7 @@ from click.testing import CliRunner
 import pytest
 
 import substra
+from substra.sdk import models
 from substra.cli.interface import cli, error_printer
 
 from . import datastore
@@ -52,7 +53,9 @@ def client_execute(directory, command, exit_code=0):
     # force using a new config file and a new profile
     if '--config' not in command:
         cfgpath = directory / 'substra.cfg'
-        substra.sdk.config.Manager(str(cfgpath)).add_profile('default', url='http://foo')
+        manager = substra.sdk.config.ConfigManager(str(cfgpath))
+        manager.set_profile('default', url='http://foo')
+        manager.save()
         command.extend(['--config', str(cfgpath)])
     return execute(command, exit_code=exit_code)
 
@@ -86,7 +89,7 @@ def test_command_config(workdir):
     # has been created
     with cfgfile.open() as fp:
         cfg = json.load(fp)
-    expected_profiles = ['default', 'foo']
+    expected_profiles = ['foo']
     assert list(cfg.keys()) == expected_profiles
 
 
@@ -101,29 +104,29 @@ def test_command_login(workdir, mocker):
     m.assert_called()
 
 
-@pytest.mark.parametrize('asset_name,key_field', [
-    ('objective', 'key'),
-    ('dataset', 'key'),
-    ('algo', 'key'),
-    ('aggregate_algo', 'key'),
-    ('composite_algo', 'key'),
-    ('testtuple', 'key'),
-    ('traintuple', 'key'),
-    ('aggregatetuple', 'key'),
-    ('composite_traintuple', 'key'),
-    ('compute_plan', 'computePlanID'),
+@pytest.mark.parametrize('asset_name,key_field,model', [
+    ('objective', 'key', models.Objective),
+    ('dataset', 'key', models.Dataset),
+    ('algo', 'key', models.Algo),
+    ('aggregate_algo', 'key', models.AggregateAlgo),
+    ('composite_algo', 'key', models.CompositeAlgo),
+    ('testtuple', 'key', models.Testtuple),
+    ('traintuple', 'key', models.Traintuple),
+    ('aggregatetuple', 'key', models.Aggregatetuple),
+    ('composite_traintuple', 'key', models.CompositeTraintuple),
+    ('compute_plan', 'compute_plan_id', models.ComputePlan),
 ])
-def test_command_list(asset_name, key_field, workdir, mocker):
-    item = getattr(datastore, asset_name.upper())
+def test_command_list(asset_name, key_field, model, workdir, mocker):
+    item = model(**getattr(datastore, asset_name.upper()))
     method_name = f'list_{asset_name}'
     m = mock_client_call(mocker, method_name, [item])
     output = client_execute(workdir, ['list', asset_name])
     m.assert_called()
-    assert item[key_field] in output
+    assert getattr(item, key_field) in output
 
 
 def test_command_list_node(workdir, mocker):
-    mock_client_call(mocker, 'list_node', datastore.NODES)
+    mock_client_call(mocker, 'list_node', [models.Node(**node) for node in datastore.NODES])
     output = client_execute(workdir, ['list', 'node'])
     assert output == ('NODE ID                     \n'
                       'foo                         \n'
@@ -148,7 +151,13 @@ def test_command_list_node(workdir, mocker):
 def test_command_add(asset_name, params, workdir, mocker):
     method_name = f'add_{asset_name}'
 
-    m = mock_client_call(mocker, method_name, response={})
+    if asset_name == 'compute_plan':
+        m = mock_client_call(mocker, method_name, response={'key': 'foo'})
+    else:
+        m = mock_client_call(mocker, method_name, response='foo')
+    item = getattr(datastore, asset_name.upper())
+    mock_client_call(mocker, f'get_{asset_name}', item)
+
     json_file = workdir / "valid_json_file.json"
     json_file.write_text(json.dumps({}))
     client_execute(workdir, ['add', asset_name] + params + [str(json_file)])
@@ -170,7 +179,10 @@ def test_command_add_objective(workdir, mocker):
     json_file = workdir / "valid_json_file.json"
     json_file.write_text(json.dumps({}))
 
-    m = mock_client_call(mocker, 'add_objective', response={})
+    m = mock_client_call(mocker, 'add_objective', response='foo')
+    item = getattr(datastore, 'OBJECTIVE')
+    m = mock_client_call(mocker, 'get_objective', item)
+
     client_execute(workdir, ['add', 'objective', str(json_file), '--dataset-key', 'foo',
                              '--data-samples-path', str(json_file)])
     m.assert_called()
@@ -205,7 +217,9 @@ def test_command_add_objective(workdir, mocker):
     (['--head-model-key', 'e', '--trunk-model-key', 'e'])
 ])
 def test_command_add_composite_traintuple(mocker, workdir, params):
-    m = mock_client_call(mocker, 'add_composite_traintuple', response={})
+    m = mock_client_call(mocker, 'add_composite_traintuple', response='foo')
+    item = getattr(datastore, 'composite_traintuple'.upper())
+    m = mock_client_call(mocker, 'get_composite_traintuple', item)
     json_file = workdir / "valid_json_file.json"
     json_file.write_text(json.dumps({}))
     client_execute(workdir, ['add', 'composite_traintuple', '--algo-key', 'foo', '--dataset-key',
@@ -231,14 +245,18 @@ def test_command_add_composite_traintuple_missing_model_key(mocker, workdir, par
     ['--in-model-key', 'bar']
 ])
 def test_command_add_aggregatetuple(mocker, workdir, params):
-    m = mock_client_call(mocker, 'add_aggregatetuple', response={})
+    m = mock_client_call(mocker, 'add_aggregatetuple', response='foo')
+    item = getattr(datastore, 'aggregatetuple'.upper())
+    m = mock_client_call(mocker, 'get_aggregatetuple', item)
     client_execute(workdir, ['add', 'aggregatetuple', '--algo-key', 'foo',
                              '--in-model-key', 'foo'] + params + ['--worker', 'foo'])
     m.assert_called()
 
 
 def test_command_add_testtuple_no_data_samples(mocker, workdir):
-    m = mock_client_call(mocker, 'add_testtuple', response={})
+    m = mock_client_call(mocker, 'add_testtuple', response='foo')
+    item = getattr(datastore, 'testtuple'.upper())
+    m = mock_client_call(mocker, 'get_testtuple', item)
     client_execute(workdir, ['add', 'testtuple', '--objective-key', 'foo',
                              '--traintuple-key', 'foo'])
     m.assert_called()
@@ -290,25 +308,25 @@ def test_command_add_data_sample_already_exists(workdir, mocker):
     m.assert_called()
 
 
-@pytest.mark.parametrize('asset_name,key_field', [
-    ('objective', 'key'),
-    ('dataset', 'key'),
-    ('algo', 'key'),
-    ('aggregate_algo', 'key'),
-    ('composite_algo', 'key'),
-    ('testtuple', 'key'),
-    ('traintuple', 'key'),
-    ('aggregatetuple', 'key'),
-    ('composite_traintuple', 'key'),
-    ('compute_plan', 'computePlanID'),
+@pytest.mark.parametrize('asset_name,key_field,model', [
+    ('objective', 'key', models.Objective),
+    ('dataset', 'key', models.Dataset),
+    ('algo', 'key', models.Algo),
+    ('aggregate_algo', 'key', models.AggregateAlgo),
+    ('composite_algo', 'key', models.CompositeAlgo),
+    ('testtuple', 'key', models.Testtuple),
+    ('traintuple', 'key', models.Traintuple),
+    ('aggregatetuple', 'key', models.Aggregatetuple),
+    ('composite_traintuple', 'key', models.CompositeTraintuple),
+    ('compute_plan', 'compute_plan_id', models.ComputePlan),
 ])
-def test_command_get(asset_name, key_field, workdir, mocker):
-    item = getattr(datastore, asset_name.upper())
+def test_command_get(asset_name, key_field, model, workdir, mocker):
+    item = model(**getattr(datastore, asset_name.upper()))
     method_name = f'get_{asset_name}'
     m = mock_client_call(mocker, method_name, item)
     output = client_execute(workdir, ['get', asset_name, 'fakekey'])
     m.assert_called()
-    assert item[key_field] in output
+    assert getattr(item, key_field) in output
 
 
 def test_command_describe(workdir, mocker):
@@ -326,7 +344,11 @@ def test_command_download(workdir, mocker):
 
 
 def test_command_cancel_compute_plan(workdir, mocker):
-    m = mock_client_call(mocker, 'cancel_compute_plan', datastore.COMPUTE_PLAN)
+    m = mock_client_call(
+        mocker,
+        'cancel_compute_plan',
+        models.ComputePlan(**datastore.COMPUTE_PLAN)
+    )
     client_execute(workdir, ['cancel', 'compute_plan', 'fakekey'])
     m.assert_called()
 
@@ -338,7 +360,7 @@ def test_command_leaderboard(workdir, mocker):
 
 
 def test_command_update_dataset(workdir, mocker):
-    m = mock_client_call(mocker, 'update_dataset')
+    m = mock_client_call(mocker, 'link_dataset_with_objective')
     client_execute(workdir, ['update', 'dataset', 'key1', 'key2'])
     m.assert_called()
 
